@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from os.path import isfile as _isfile, dirname as _dirname, abspath as _abspath, basename as _basename
+from os import rename as _rename
 from copy import copy as _copy
 import re as _re
 from json import load as _load, dump as _dump
@@ -7,9 +8,8 @@ from json import load as _load, dump as _dump
 import ipaddress as _ipaddress
 from datetime import datetime as _datetime
 
-from creole.loader import creole_loader as _creole_loader
 from tiramisu.option import DomainnameOption as _DomainnameOption
-from time import time
+from creole.loader import creole_loader as _creole_loader
 
 _CONFIG_FILE = '/var/lib/eole/config/eoledhcp.cfg'
 
@@ -49,6 +49,7 @@ def _empty_dhcp_json():
             'dyn': [],
             'ip': [],
             'name': [],
+            'all_ranges': [],
             }
 
 
@@ -72,8 +73,11 @@ def _get_loader(mandatory_permissive=True):
                     else:
                         new_json['ip'].append(None)
                     new_json['name'].append(root_cfg['dhcp.dhcp.id_dhcp.name']['val'][idx])
+                    new_json['all_ranges'].append(False)
             except:
                 new_json = _empty_dhcp_json()
+            finally:
+                _rename('/etc/eole/extra/dhcp/config.eol', '/etc/eole/extra/dhcp/config.eol.bak')
         _dump(new_json, open('/etc/eole/dhcp.json', 'w'))
     return _creole_loader(load_extra=True, rw=True, owner=_MODNAME,
                          mandatory_permissive=mandatory_permissive, force_dirs=_force_dirs,
@@ -106,7 +110,7 @@ def _extract_gen_config_data(cfg):
                                [_ipaddress.ip_address(ip)
                                 for ip in list(cfg.creole.dhcp.adresse_network_dhcp.ip_haute_dhcp)
                                ],
-                               list(cfg.creole.dhcp.adresse_network_dhcp.interdire_hotes_inconnus),
+                               [op if op in ['oui', 'non'] else 'oui' for op in cfg.creole.dhcp.adresse_network_dhcp.interdire_hotes_inconnus],
                                [True if s == 'oui' else False
                                 for s in list(cfg.creole.dhcp.adresse_network_dhcp.adressage_statique)]
                                ))
@@ -182,7 +186,7 @@ def _validate_rangename(host_rangename, ranges):
         if ranges[range_name]['restricted'] is False:
             raise ValueError('La plage "{}" ne peut pas contenir d’IP réservée'.format(range_name))
     else:
-        raise ValueError("Le nom {} ne correspond à aucune plage".format(host_rangename))
+        raise ValueError('Le nom "{}" ne correspond à aucune plage'.format(host_rangename))
 
 
 # __________________________________________________________
@@ -201,12 +205,13 @@ def _get_config(name):
         return cfg.get('eole', 'dhcp_path')
 
 
-_LEASES_DATABASE = '/var/lib/dhcp/dhcpd.leases'
+_LEASES_DATABASE = _get_config("LEASES_FILE")
+
 _DHCP_LEASES_ATTRS = {
         'ends': _re.compile(r'^\s*ends\s+(?P<value>.*);', _re.M),
         'binding state': _re.compile(r'^\s*binding state\s+(?P<value>.*);', _re.M),
         'hardware ethernet': _re.compile(r'^\s*hardware ethernet\s+(?P<value>[a-f0-9]{2}(:[a-f0-9]{2}){5});', _re.M),
-        'client-hostname': _re.compile(r'^\s*client-hostname\s+"?(?P<value>\w+)"?;', _re.M)
+        'client-hostname': _re.compile(r'^\s*client-hostname\s+"?(?P<value>[\w-]+)"?;', _re.M)
         }
 _leases_re = _re.compile(r'^(?P<lease>lease (?P<ip>[0-9]{1,3}(?:\.[0-9]{1,3}){3})\s+\{.*?\})', _re.M|_re.S)
 
@@ -260,10 +265,11 @@ def get_leases(*args, **kwargs):
         parse_lease = _parse_attrs(lease.group('lease'))
         name = parse_lease.setdefault('client-hostname', None)
         if name and parse_lease['binding state'] == 'active':
+            name = name.lower()
             try:
                 _DomainnameOption('test', '', name, type_='domainname', allow_ip=False, allow_without_dot=True, allow_uppercase=True)
             except ValueError:
-                lease['client-hostname'] = None
+                name = None
             ip = lease.group('ip')
             expiration = parse_lease.get('ends')
             if not expiration:
@@ -275,7 +281,7 @@ def get_leases(*args, **kwargs):
             else:
                 expiration = expiration.split(None, 1)[1]
             leases[ip] = {'id': idx,
-                          'name': parse_lease['client-hostname'],
+                          'name': name,
                           'address': ip,
                           'mac': parse_lease['hardware ethernet'],
                           'expiration': expiration}
@@ -285,10 +291,13 @@ def get_leases(*args, **kwargs):
 
 def _get_free_ip(is_dynamic, total, leases, first, last):
     if is_dynamic: #free leases is the difference between total ip and active leases
+        free_leases = []
         for idx, lease in enumerate(leases):
             if first <= lease <= last:
                 total -= 1
-                del leases[idx]
+            else:
+                free_leases.append(leases[idx])
+        leases = free_leases
         return total
 
 def _get_subnets(cfg):
@@ -298,7 +307,7 @@ def _get_subnets(cfg):
     nom_plage_dhcp = list(cfg.creole.dhcp.adresse_network_dhcp.nom_plage_dhcp)
     ip_haute_dhcp = list(cfg.creole.dhcp.adresse_network_dhcp.ip_haute_dhcp)
     ip_basse_dhcp = list(cfg.creole.dhcp.adresse_network_dhcp.ip_basse_dhcp)
-    interdire_hotes_inconnus = list(cfg.creole.dhcp.adresse_network_dhcp.interdire_hotes_inconnus)
+    interdire_hotes_inconnus = [op if op in ['oui', 'non'] else 'oui' for op in cfg.creole.dhcp.adresse_network_dhcp.interdire_hotes_inconnus]
     range_type = list(cfg.creole.dhcp.adresse_network_dhcp.adressage_statique)
     leases = [int(_ipaddress.ip_address(lease['address'])) for lease in get_leases()['leases']]
     for idx, network in enumerate(adresse_network_dhcp):
@@ -373,7 +382,7 @@ def get_subnets(*args, **kwargs):
 # reservation
 
 
-def _get_reserved(cfg, reserved_id, rel):
+def _get_reserved(cfg, reserved_id, rel, remove_all=False):
     def _get_host_by_id(id_dh, idx):
         dico = dict(id=id_dh, hostname=hostname[idx], macaddress=macaddress[idx])
         if dyn[idx] == 'oui':
@@ -382,6 +391,8 @@ def _get_reserved(cfg, reserved_id, rel):
             if name[idx]:
                 dico['rangename'] = name[idx]
             dico['ip'] = ip[idx]
+        if not remove_all:
+            dico['all_ranges'] = all_ranges[idx]
         return dico
     root_cfg = _get_json_config()
     id_dhcp = root_cfg['id_dhcp']
@@ -390,6 +401,8 @@ def _get_reserved(cfg, reserved_id, rel):
     dyn = root_cfg['dyn']
     ip = root_cfg['ip']
     name = root_cfg['name']
+    if not remove_all:
+        all_ranges = root_cfg['all_ranges']
     reserved = []
     messages = []
     if reserved_id is not None:
@@ -423,11 +436,18 @@ def get_reserved(reserved_id=None, *args, **kwargs):
 
 
 def _get_json_config():
-    return _load(open('/etc/eole/dhcp.json'))
+    load = _load(open('/etc/eole/dhcp.json'))
+    # add 'all_ranges' column if not exists
+    if 'all_ranges' not in load:
+        load['all_ranges'] = []
+        for id_dhcp in load['id_dhcp']:
+            load['all_ranges'].append(False)
+
+    return load
 
 
 @_check_activation
-def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=False, del_reservations=False, *args, **kwargs):
+def upsert_reserved(reserved, deleted=[], hosts_range=None, prefix=None, save_when_error=False, del_reservations=False, valid_conflit_other_range=True, force_new=False, *args, **kwargs):
     """
     Creates or modifies an existing DHCP bookings
     :param dict kwargs:  - with the 'id' key -> update
@@ -454,18 +474,17 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
             low = min(candidates_ips)
             high = max(candidates_ips)
         if not dispendable_ips:
-            raise ValueError('Pas d’adresse IP disponible dans la plage "{}-{}" pour {}'.format(low, high, mac))
+            raise ValueError('Pas d’adresse IP disponible dans la plage "{}-{}"'.format(low, high))
         else:
             ip = sorted(list(dispendable_ips)).pop(0)
             return _ipaddress.ip_address(ip)
 
-    def _make_name(hosts_range):
-        if set(['rangename']) != set(hosts_range.keys()):
-            raise ValueError("Le dictionnaire du nom de la plage est invalide, il faut la clef 'rangename'")
-        return hosts_range['rangename']
+#    def _make_name(hosts_range):
+#        if set(['rangename']) != set(hosts_range.keys()):
+#            raise ValueError("Le dictionnaire du nom de la plage est invalide, il faut la clef 'rangename'")
+#        return hosts_range['rangename']
 
-    def _make_hostname(prefix, hostnames, reuse=True):
-        auto_hostname_re = _re.compile(r'{}(?P<suffixe>[0-9]+)'.format(prefix))
+    def _make_hostname(prefix, hostnames, auto_hostname_re, reuse=True):
         hostnames_with_prefix = [pref for pref in hostnames if pref is not None and pref.startswith(prefix)]
         suffixes = []
         for hostname_with_prefix in hostnames_with_prefix:
@@ -486,6 +505,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
         new_name = '{}{}'.format(prefix, suffixe)
         return new_name
 
+    auto_hostname_re = None
     # retrieving the existing booking list in the cfg.dhcp.dhcp config variable
     messages = []
     try:
@@ -502,6 +522,8 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
 
         if del_reservations:
             _del_reserved(cfg, messages, None, True)
+        elif deleted:
+            _del_reserved(cfg, messages, deleted, False)
         if messages != []:
             pass
 
@@ -523,10 +545,11 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
             # Fetch known reservations
             known_id_dhcp_global = root_cfg['id_dhcp']
             known_hostnames_global = root_cfg['hostname']
-            known_macaddresses_global = root_cfg['macaddress']
+            known_macaddresses_global = [mc.lower() for mc in root_cfg['macaddress']]
             known_dyns_global = root_cfg['dyn']
             known_ips_global = root_cfg['ip']
             known_rangenames_global = root_cfg['name']
+            known_all_ranges = root_cfg['all_ranges']
             for host in reserved:
                 # validation of the inner structure
                 if not isinstance(host, dict):
@@ -539,6 +562,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                 # Informations from host
                 macaddress = host.get('macaddress')
                 targeted_range = host.get('rangename')
+                all_ranges = host.get('all_ranges', False)
                 ip = host.get('ip')
                 hostname = host.get('hostname')
                 if hostname is not None:
@@ -549,7 +573,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
 
                 # Find if this is new or update case
                 c_id = host.get('id')
-                if c_id is None:
+                if c_id is None or force_new:
                     last_id += 1
                     c_id = last_id
                     new = True
@@ -571,6 +595,8 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                                         'msg': "L’adresse MAC {} n’est pas valide".format(macaddress),
                                         'rel': host})
                         continue
+                    else:
+                        macaddress = macaddress.lower()
                 else:
                     messages.append({'level': 'error',
                                      'msg': "L’adresse MAC est obligatoire pour une réservation",
@@ -604,7 +630,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                 # Range name and subnet
                 if targeted_range and targeted_range not in ranges:
                     messages.append({'level': 'error',
-                                     'msg': "Le nom {} ne correspond à aucune plage".format(targeted_range),
+                                     'msg': 'Le nom "{}" ne correspond à aucune plage'.format(targeted_range),
                                      'rel': host})
                     continue
                 if targeted_range:
@@ -614,17 +640,17 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                 # la plage est valide si elle est incluse dans l’ensemble des IP attribuables
                 auto_range = set()
                 if hosts_range is not None:
-                    if hosts_range.get('rangename', None) is not None:
-                        range_desc = ranges[hosts_range['rangename']]
-                        if range_desc['static'] is False and range_desc['restricted'] is False:
-                            messages.append({'level': 'error',
-                                            'msg': "La plage n’accepte pas de réservation",
-                                            'rel': host})
-                            continue
-                        host['rangename'] = hosts_range['rangename']
-                        targeted_range = hosts_range['rangename']
-                        targeted_subnet = ranges[targeted_range].get('subnet')
-                    elif hosts_range.get('low', None) is not None and hosts_range.get('high', None) is not None:
+                    #if hosts_range.get('rangename', None) is not None:
+                    #    range_desc = ranges[hosts_range['rangename']]
+                    #    if range_desc['static'] is False and range_desc['restricted'] is False:
+                    #        messages.append({'level': 'error',
+                    #                        'msg': "La plage n’accepte pas de réservation",
+                    #                        'rel': host})
+                    #        continue
+                    #    host['rangename'] = hosts_range['rangename']
+                    #    targeted_range = hosts_range['rangename']
+                    #    targeted_subnet = ranges[targeted_range].get('subnet')
+                    if hosts_range.get('low', None) is not None and hosts_range.get('high', None) is not None:
                         try:
                             range_floor = _ipaddress.ip_address(hosts_range['low'])
                             range_ceil = _ipaddress.ip_address(hosts_range['high'])
@@ -706,29 +732,45 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                     host['rangename'] = targeted_range
 
                 if not hostname:
-                    hostname = _make_hostname(prefix, known_hostnames_global)
+                    if auto_hostname_re is None:
+                        auto_hostname_re = _re.compile(r'{}(?P<suffixe>[0-9]+)'.format(prefix))
+                    hostname = _make_hostname(prefix, known_hostnames_global, auto_hostname_re)
 
                 # tests complémentaires sur les valeurs :
-                if known_id_dhcp_global:
-                    macaddress_reservations = ({'ip': kr[2], 'rangename': kr[3]}
-                                               for kr in zip(known_macaddresses_global, known_id_dhcp_global, known_ips_global, known_rangenames_global)
-                                               if macaddress == kr[0] and c_id != kr[1])
-
-                    if macaddress_reservations:
-                        for resa in macaddress_reservations:
-                            conflicting_subnet, conflicting_range = _subnet_host_is_in(resa, subnets, ranges)
-                            if conflicting_subnet == targeted_subnet:
-                                if conflicting_range:
-                                    if ranges[conflicting_range]['static']:
-                                        conflict_desc = 'a déjà une adresse IP réservée dans la plage statique {}'.format(conflicting_range)
-                                    else:
-                                        conflict_desc = 'est déjà associé à la plage dynamique {}'.format(conflicting_range)
-                                else:
-                                    conflict_desc = 'a déjà une adresse IP réservée hors plage'
+                if known_id_dhcp_global and (valid_conflit_other_range or all_ranges):
+                    for mac_idx, mac in enumerate(known_macaddresses_global):
+                        if mac == macaddress and known_rangenames_global[mac_idx] == targeted_range:
+                            messages.append({'level': 'error',
+                                             'msg': "L’hôte {} ({}) est déjà associé à la plage {}".format(hostname, macaddress, targeted_range),
+                                             'rel': host})
+                            continue
+                if known_id_dhcp_global and valid_conflit_other_range:
+                    if all_ranges:
+                        for mac_idx, mac in enumerate(known_macaddresses_global):
+                            if mac == macaddress and known_rangenames_global[mac_idx] == hosts_range:
                                 messages.append({'level': 'error',
-                                                 'msg': "L’hôte {} ({}) {} dans le réseau {}".format(hostname, macaddress, conflict_desc, str(conflicting_subnet)),
+                                                 'msg': "L’hôte {} ({}) {} est déjà associé à la plage {}".format(hostname, macaddress, hosts_range),
                                                  'rel': host})
                                 continue
+                    else:
+                        macaddress_reservations = ({'ip': kr[2], 'rangename': kr[3]}
+                                                   for kr in zip(known_macaddresses_global, known_id_dhcp_global, known_ips_global, known_rangenames_global)
+                                                   if macaddress == kr[0] and c_id != kr[1])
+                        if macaddress_reservations:
+                            for resa in macaddress_reservations:
+                                conflicting_subnet, conflicting_range = _subnet_host_is_in(resa, subnets, ranges)
+                                if conflicting_subnet == targeted_subnet:
+                                    if conflicting_range:
+                                        if ranges[conflicting_range]['static']:
+                                            conflict_desc = 'a déjà une adresse IP réservée dans la plage statique {}'.format(conflicting_range)
+                                        else:
+                                            conflict_desc = 'est déjà associé à la plage dynamique {}'.format(conflicting_range)
+                                    else:
+                                        conflict_desc = 'a déjà une adresse IP réservée hors plage'
+                                    messages.append({'level': 'error',
+                                                     'msg': "L’hôte {} ({}) {} dans le réseau {}".format(hostname, macaddress, conflict_desc, str(conflicting_subnet)),
+                                                     'rel': host})
+                                    continue
 
                 try:
                     if not new:
@@ -758,6 +800,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
                             else:
                                 known_rangenames_global.append(None)
                             known_ips_global.append(str(ip))
+                        known_all_ranges.append(all_ranges)
                 except ValueError as err:
                     messages.append({'level': 'error', 'msg': str(err),
                                      'rel': host})
@@ -769,6 +812,7 @@ def upsert_reserved(reserved, hosts_range=None, prefix=None, save_when_error=Fal
             root_cfg['dyn'] = known_dyns_global
             root_cfg['ip'] = known_ips_global
             root_cfg['name'] = known_rangenames_global
+            root_cfg['all_ranges'] = known_all_ranges
             root_cfg['last_id'] = last_id
             # applying the new DHCP configuration
             # do not save on errors, unless save_when_error is True
@@ -789,7 +833,9 @@ def _del_reserved(cfg, messages, reserved, allreservations):
         if isinstance(reserved, list):
             root_cfg = _get_json_config()
             for res in reserved:
-                orig_reserved = _get_reserved(cfg, res['id'], res)['reserved']
+                if 'all_ranges' in res:
+                    del res['all_ranges']
+                orig_reserved = _get_reserved(cfg, res['id'], res, remove_all=True)['reserved']
                 if orig_reserved and res != orig_reserved[0]:
                     messages.append({'level': 'error',
                                      'msg': 'host with id {} has not '
